/*
 *  video_out_sdl.c
 *
 *  Copyright (C) Ryan C. Gordon <icculus@lokigames.com> - April 22, 2000.
 *
 *  A mpeg2dec display driver that does output through the
 *  Simple DirectMedia Layer (SDL) library. This effectively gives us all
 *  sorts of output options: X11, SVGAlib, fbcon, AAlib, GGI. Win32, MacOS
 *  and BeOS support, too. Yay. SDL info, source, and binaries can be found
 *  at http://slouken.devolution.com/SDL/
 *
 *  This file is part of mpeg2dec, a free MPEG-2 video stream decoder.
 *	
 *  mpeg2dec is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  mpeg2dec is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <SDL/SDL.h>
#include "lvedes.h"
#include "display.h"

static SDL_Surface *screen = NULL;
static SDL_Surface *bmp = NULL;
static SDL_Surface *bmp_sav = NULL;
static SDL_Overlay *overlay = NULL;
static SDL_Rect dispSize;
static SDL_Rect dispSize2;
static Uint8 *keyState = NULL;
static int framePlaneY = -1;
static int framePlaneUV = -1;


static inline int 
findArrayEnd(SDL_Rect **array)
/*
 * Take a null-terminated array of pointers, and find the last element.
 *
 *    params : array == array of which we want to find the last element.
 *   returns : index of last NON-NULL element.
 */
{
    int i;
    for (i = 0; array[i] != NULL; i++);  // keep loopin'...
    return(i - 1);
} // findArrayEnd

SDL_Overlay* display_init(T_VIDEO *video, char *title, int scene_flag)
/*
 * Initialize an SDL screen and an SDL YUV overlay.
 *
 *    params : width  == width of video we'll be displaying.
 *             height == height of video we'll be displaying.
 *             fullscreen == want to be fullscreen?
 *             title == Title for window titlebar.
 *   returns : non-zero on success, zero on error.
 */
{
    int rc = 0;
    int i = 0;
    char fname[1024];
    const SDL_VideoInfo *vidInfo = NULL;
    int desiredWidth = -1;
    int desiredHeight = -1;
    SDL_Rect **modes = NULL;
    Uint32 sdlflags = SDL_HWSURFACE;
    Uint8 bpp;
    
    SDL_Rect dst_rect;
    
    
    rc = SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
    if (rc != 0)
    {
        printf("SDL: SDL_Init() failed! rc == (%d).\n", rc);
        return(NULL);
    } 
    atexit(SDL_Quit);

    vidInfo = SDL_GetVideoInfo();

    modes = SDL_ListModes(vidInfo->vfmt, sdlflags);
    if (modes == NULL)
    {
      printf("SDL modes w/o HWSURFACE\n");
      sdlflags &= ~SDL_HWSURFACE;
      modes = SDL_ListModes(vidInfo->vfmt, sdlflags);   // give me ANYTHING.
      if (modes == NULL)
      {
        printf("SDL: SDL_ListModes() failed.\n");
        return(NULL);
      } // if
    
      for (i=0; modes[i]; i++)
        printf("   %d x %d\n", modes[i]->w, modes[i]->h);
    }
    else if (modes == (SDL_Rect **)-1)   // anything is fine.
    {
      desiredWidth = 800;
      desiredHeight = 768;
    } 
    else
    {
        // we want to get the lowest resolution that'll fit the video.
        //  ...so start at the far end of the array.
        for (i = findArrayEnd(modes); ((i >= 0) && (desiredWidth == -1)); i--)
        {
            if ((modes[i]->w >= video->w) && (modes[i]->h >= video->h))
            {
                desiredWidth = modes[i]->w;
                desiredHeight = modes[i]->h;
            } 
        } 
    } 

    if ((desiredWidth < 0) || (desiredHeight < 0))
    {
        printf("SDL: Couldn't produce a mode with at least"
               " a (%dx%d) resolution!\n", video->w, video->h);
        return(NULL);
    } 

    //-- Show Cursor --
    SDL_ShowCursor(1);

        // YUV overlays need at least 16-bit color depth, but the
        //  display might less. The SDL AAlib target says it can only do
        //  8-bits, for example. So, if the display is less than 16-bits,
        //  we'll force the BPP to 16, and pray that SDL can emulate for us.
    bpp = vidInfo->vfmt->BitsPerPixel;
    if (bpp < 16)
    {
        printf("\n\n"
               "WARNING: Your SDL display target wants to be at a color\n"
               " depth of (%d), but we need it to be at least 16 bits,\n"
               " so we need to emulate 16-bit color. This is going to slow\n"
               " things down; you might want to increase your display's\n"
               " color depth, if possible.\n\n", bpp);
        bpp = 16;  // (*shrug*)
    } // if
 
    strcpy(fname, PATH_LIB);
    strcat(fname, FN_SCREEN_BMP);
     
    if ((bmp = SDL_LoadBMP(fname)) == NULL)
    {
      printf("ERROR: SDL could not load screen bitmap (%s)\n", fname);
    }
    else
    {
      desiredWidth = bmp->w;
      desiredHeight = bmp->h;
    }
     
    screen = SDL_SetVideoMode(desiredWidth, desiredHeight, bpp, sdlflags);
    if (screen == NULL)
    {
      printf("ERROR: SDL could not set the video mode!\n");
      return(NULL);
    } 
    video->screen = screen;
    video->bmp_y  = screen->h;

    dst_rect.x = 0;
    dst_rect.y = 0;
    dst_rect.w = screen->w;
    dst_rect.h = screen->h;
     
    SDL_BlitSurface(bmp, NULL, screen, &dst_rect);
    SDL_FreeSurface(bmp);
    
    if (title == NULL) title = "Help! I'm trapped inside a Palm IIIc!";

    SDL_WM_SetCaption(title, "LVE");

    dispSize.x = 708;
    dispSize.y = 48;
    dispSize.w = 288;
    dispSize.h = 216;

    /*
    overlay = SDL_CreateYUVOverlay(video->w, video->h, SDL_IYUV_OVERLAY, screen);
    if (overlay == NULL)
    {
        printf("ERROR: Couldn't create an SDL-based YUV overlay!\n");
        return(NULL);
    } // if

    video->ov   = overlay;
    video->page = overlay->pixels;

    framePlaneY = (video->w * video->h);
    framePlaneUV = ((video->w / 2) * (video->h / 2));

    printf ("Surface locking needed (%d)\n", SDL_MUSTLOCK(screen));
    */ 
    return(overlay);  // non-zero == SUCCESS. Oooh yeah.

} // display_init


static SDL_Overlay *overlay2 = NULL;
void display_scene(T_VIDEO *video)
{
  Uint8 **tmp;
  
  if (overlay2)
  {
    tmp = overlay2->pixels;
    overlay2->pixels = video->page;
    
    // SDL_LockYUVOverlay(overlay2);
    // SDL_UnlockYUVOverlay(overlay2);

    /*
    memcpy(dst[0], video->page[0], framePlaneY);
    memcpy(dst[1], src[1], framePlaneUV);
    memcpy(dst[2], src[2], framePlaneUV);
    */
    
    SDL_DisplayYUVOverlay(overlay2, &dispSize2);
    
    overlay2->pixels = tmp;
  }  
} 

void display_scale(T_VIDEO *video, int zoom)
{
  return;

  video->refresh_bg = 1;
  
  switch (zoom)
  {
    //-- half --
    case 0:
      dispSize.x = screen->w/2 - video->w/4;
      dispSize.y = video->bmp_y/2 - video->h/4;    
      dispSize.w = video->w>>1;
      dispSize.h = video->h>>1;
      break;

    //-- double --
    case 2:
      if (video->w < 512 && video->h < 384)
      { 
        dispSize.x = screen->w/2  - video->w;
        dispSize.y = video->bmp_y/2 - video->h;
        dispSize.w = video->w<<1;
        dispSize.h = video->h<<1;
      }  
      break;  
      
    //-- original --  
    default:  
      dispSize.x = (screen->w - video->w)/2;
      dispSize.y = (video->bmp_y - video->h)/2;
      dispSize.w = video->w;
      dispSize.h = video->h;
      break;  
  }            
} 

Uint32 display_draw_frame(T_VIDEO *video)
/*
 * Draw a frame to the SDL YUV overlay.
 *
 *   params : *src[] == the Y, U, and V planes that make up the frame.
 *  returns : non-zero on success, zero on error.
 */
{
    //SDL_DisplayYUVOverlay(overlay, &dispSize);
    return(-1);
}

void display_lock()
{
  while (SDL_LockYUVOverlay(overlay) < 0);
}

void display_unlock()
{
  SDL_UnlockYUVOverlay(overlay);
}  

void display_refresh(T_VIDEO *video)
{
  SDL_UpdateRect(screen, 0, 0, 0, 0);  
  video->refresh_bg = 0;
}

static SDL_Rect slider0 = {0,0,0,0};
void display_slider(T_VIDEO *video, SDL_Rect *slider)
{
  return;
  
  slider0.x = slider->x - 10 + video->act_pos * slider->w;
  slider0.y = slider->y;
  slider0.w = slider->w;
  slider0.h = slider->h;

  SDL_BlitSurface(screen, &slider0, bmp_sav, NULL);
  SDL_BlitSurface(bmp, NULL, screen, &slider0);
  SDL_UpdateRect(screen, slider->x, slider->y, slider->w, slider->h);
  SDL_BlitSurface(bmp_sav, NULL, screen, &slider0);
}
